<?php

namespace Drupal\tmgmt_cdt\Entity;

use Drupal\Component\Serialization\Json;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\tmgmt\Entity\JobItem;
use Drupal\tmgmt\TMGMTException;
use Drupal\tmgmt_cdt\TmgmtCdtFormatHelper;

/**
 * Override classe JobItem.
 */
class TmgmtCdtJobItem extends JobItem {

  use StringTranslationTrait;

  /**
   * For job items in cart without job attached.
   *
   * @var bool
   */
  public $onCart = FALSE;

  /**
   * {@inheritdoc}
   */
  public function isAbortable() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function abortTranslation() {
    throw new TMGMTException('Function disabled. Please check the CdT documentation');
  }

  /**
   * {@inheritdoc}
   */
  public function addTranslatedData(array $translation, $key = [], $status = NULL) {

    if ($this->isInactive()) {
      // The job item can not be inactive and receive translations.
      $this->setState(JobItemInterface::STATE_ACTIVE);
    }

    $this->addTranslatedDataRecursive($translation, $key, $status);
    // Check if the job item has all the translated data that it needs now.
    // Only attempt to change the status to needs review if it is currently
    // active.
    if ($this->isActive()) {
      $data = \Drupal::service('tmgmt.data')->filterTranslatable($this->getData());
      $finished = TRUE;
      foreach ($data as $item) {
        if (empty($item['#status']) || $item['#status'] == TMGMT_DATA_ITEM_STATE_PENDING || $item['#status'] == TMGMT_DATA_ITEM_STATE_PRELIMINARY) {
          $finished = FALSE;
          break;
        }
      }
      if ($finished && $this->getJob()->hasTranslator()) {
        // There are no unfinished elements left.
        if ($this->getJob()->getTranslator()->isAutoAccept()) {
          // If the job item is going to be auto-accepted, set to review without
          // a message.
          $this->needsReview(FALSE);
        }
        else {
          // Otherwise, create a message that contains source label, target
          // language and links to the review form.
          $job_url = $this->getJob()->toUrl()->toString();
          $variables = [
            '@source' => $this->getSourceLabel(),
            '@language' => $this->getJob()->getTargetLanguage()->getName(),
            ':review_url' => $this->toUrl('canonical', ['query' => ['destination' => $job_url]])->toString(),
          ];
          (!$this->getSourceUrl()) ? $variables[':source_url'] = (string) $job_url : $variables[':source_url'] = $this->getSourceUrl()->toString();
          $this->needsReview('The translation of <a href=":source_url">@source</a> to @language is finished and can now be <a href=":review_url">reviewed</a>.', $variables);
        }
      }
    }
    $this->save();
  }

  /**
   * Override parent function.
   */
  public function acceptTranslation() {
    try {
      // To not break the tmgmt logic (source and target language at job level)
      $job = $this->getJob();
      $job->set('target_language', $this->get('cdt_target_language')->value);
      $job->save();

      $result = parent::acceptTranslation();

      if (tmgmt_job_check_finished($job->id())) {
        $job->finished();
      }

      // And back to multiple.
      $job->set('target_language', 'multiple');
      $job->save();

      return $result;

    }
    catch (\Exception $e) {
      \Drupal::messenger()->deleteAll();
      \Drupal::messenger()->addError(
            $this->t(
                'Error accepting translation of Job jobItem @tjid: @error',
                [' @tjid' => $this->id(), '@error' => $e->getMessage()]
            )
            );

    }

  }

  /**
   * Override parent function.
   */
  protected function count(&$item) {
    @parent::count($item);
    if (!empty($item['#text'])) {
      if (\Drupal::service('tmgmt.data')->filterData($item)) {
        $cdt_character_count = $this->get('cdt_character_count')->value;
        $cdt_character_count += TmgmtCdtFormatHelper::characterCount($item['#text']);
        $this->set('cdt_character_count', $cdt_character_count);

        $cdt_settings = Json::decode($this->get('cdt_settings')->value);
        if (!isset($cdt_settings['links'])) {
          $cdt_settings['links'] = [];
        }
        $cdt_settings['links'] = array_merge($cdt_settings['links'], TmgmtCdtFormatHelper::extractHrefs($item['#text']));
        $this->set('cdt_settings', Json::encode($cdt_settings));
      }
    }
  }

  /**
   * Override parent function.
   */
  public function recalculateStatistics() {

    if (!$this->isAccepted()) {
      $this->set('cdt_character_count', 0);
    }
    parent::recalculateStatistics();
  }

  /**
   * {@inheritdoc}
   */
  public function getJob() {
    $job = parent::getJob();
    if (!$job && $this->onCart) {
      return new FakeTmgmtCdtJob();
    }
    if ($job) {
      $job->set('target_language', $this->get('cdt_target_language')->value);
    }

    return $job;
  }

}
